
/* cpdisp: CHROMA program display routines

>09/15/84  JLM: 1.0 original
>03/08/85  JLM: 1.0 added link_flag
>03/20/96  JLM: 1.1 replace BIOS with library calls for CRT output
>01/31/96  JLM: 1.3 Add ability to blank non-contributing parameters
                    Remove logic to not draw unchanged parameters,
                    needed for previous change.  This was originally
                    for performance.

*/

#include <conio.h>
#include <dos.h>
#include <mem.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>



#define NUMLABELS 66           /* number of crt mask labels  */

#define ALL_PARAMS                     1
#define CHANGED_PARAMS                 0

#define TRUE                           1
#define FALSE                          0

#define DISPLAY                        1
#define BLANK                          0


void disppar( int par, int val, int blanking);


void at(int row,int column)         /* set cursor position */

{
		gotoxy(column, row);
}



void clear_crt()                   

{
		clrscr();
}


typedef struct { char char_ea[32]; } strg32;
typedef struct { char char_ea[25]; } strg25;
typedef struct { char char_ea[20]; } strg20;
typedef struct { char char_ea[15]; } strg15;
typedef struct { char char_ea[14]; } strg14;
typedef struct { char char_ea[12]; } strg12;
typedef struct { char char_ea[11]; } strg11;
typedef struct { char char_ea[10]; } strg10;
typedef struct { char char_ea[ 7]; } strg7;

		 strg32 patch[16]
					  = { "split - 16 channel             ",
							"independent channel            ",
							"independent channel - synch    ",
							"independent channel - ring mod ",
							"independent channel - filter FM",
							"parallel filter                ",
							"parallel filter - synch        ",
							"parallel filter - ring mod     ",
							"parallel filter - filter FM    ",
							"series filter                  ",
							"series filter - synch          ",
							"series filter - ring mod       ",
							"series filter - filter FM      ",
							"variable mix filter            ",
							"variable mix filter - synch    ",
							"variable mix filter - filter FM"  };



       strg20 foots[8]
                = { "latch / sustain    ",
                    "latch / disabled   ",
                    "disabled / sustain ",
						  "disabled           ",
						  "gate / sustain     ",
                    "gate / disabled    ",
                    "inv gate / sustain ",
						  "inv gate / disabled"  };



       strg25 keyalg[16]
                  = { "polyphonic order played ",
                      "polyphonic pitch ordered",
                      "polyphonic chord buffer ",
                      "polyphonic all channels ",
                      "monophonic all channels ",
                      "monophonic last note S/T",
                      "monophonic last note M/T",
                      "monophonic first note   ",
                      "monophonic lowest note  ",
							 "monophonic highest note ",
							 "monophonic arpegio up   ",
                      "monophonic arpegio down ",
                      "monophonic arp up / down",
							 "monophonic arp down / up",
							 "monophonic sequencer    ",
							 "monophonic arp random   "  };


       strg11 ratmod[16]
                  = { "          ",
                      "pressure  ",
                      "+ keyboard",
                      "- keyboard",
                      "+ velocity",
                      "- velocity",
                      "+ env 2   ",
                      "- env 2   ",
							 "+ pedal 1 ",
							 "- pedal 1 ",
                      "+ pedal 2 ",
                      "- pedal 2 ",
							 "+ lever 1 ",
							 "- lever 1 ",
							 "+ lever 2 ",
                      "- lever 2 "  };



       strg12 swave[16]
                 = { "sine       ",
                     "cosine     ",
                     "offset sine",
                     "half sine  ",
                     "triangle A ",
                     "triangle B ",
							"sawtooth   ",
							"lag square ",
                     "square     ",
                     "3 step saw ",
							"4 step tri ",
							"4 step saw ",
							"6 step tri ",
                     "6 step saw ",
                     "8 step tri ",
                     "S+H random "  };



       strg14 samod[16]
                 = { "             ",
                     "pressure     ",
                     "+ keyboard   ",
                     "- keyboard   ",
							"+ velocity   ",
							"- velocity   ",
                     "+ env 1      ",
                     "- env 1      ",
							"+ pedal 1    ",
							"- pedal 2    ",
							"lever 1      ",
                     "lever 2      ",
                     ".85 sec delay",
                     "1.3 sec delay",
                     "2.6 sec delay",
                     "5.2 sec delay"  };


       strg14 touch[8]
                = { "             ",
                    "low          ",
                    "medium low   ",
						  "medium       ",
						  "medium high  ",
                    "high         ",
                    "threshold    ",
						  "inv threshold"  };



       strg11 envmod[8]
                 = { "          ",
                     "pressure  ",
                     "+ keyboard",
                     "- keyboard",
                     "+ velocity",
                     "- velocity",
                     "pedal 1   ",
                     "pedal 2   "  };



       strg11 mod[16]
               = { "keyglide A",
						 "sweep A   ",
						 "env 1A    ",
						 "env 2A    ",
                   "keyglide B",
                   "sweep B   ",
                   "env 1B    ",
                   "env 2B    ",
                   "lever 1   ",
                   "lever 2   ",
                   "pedal 1   ",
                   "pedal 2   ",
                   "velocity  ",
                   "thresh vel",
                   "pressure  ",
						 "thresh prs"  };



		 strg10 filter[2]
					  = { "low pass ",
							"high pass"  };




       strg7 amp[4]
              = { "env 1A",
                  "env 2A",
                  "env 1B",
                  "env 2B"  };



		 strg10 ampmod[8]
                 = { "         ",
                     "pressure ",
							"keyboard ",
							"sweep    ",
							"+ pedal 1",
                     "- pedal 1",
                     "+ pedal 2",
                     "- pedal 2"  };


       strg14 sweepm[4]
                 = { "ind freerun  ",
                     "ind keyed    ",
                     "synch freerun",
                     "synch keyed  "  };


		 strg15 wave [4]
               = { "sawtooth      ",
                   "square / pulse",
						 "pink noise    ",
						 "white noise   "  };



       strg11 glide[2]
                = { "portemento",
                     "glissando "  };



typedef struct {
       int row;
       int column;
		 char *xlate;
       int blank;
		 } pct;


       /* for each parameter:
            [0] row number at which to display
            [1] column number at which to display
            [2] =0 numeric
                =1 bipolar numeric
                else discreet with poiter to table to translate
            [3] =0 always display
                =1 blank if zero (used for modulation depths, etc.)
                =2 blank if next parameter is 0 (used for modulation sources)
                =3 blank if previous parameter is 0 (used for glide shape)
       */

       pct par_crt_table [101]
							= {  0, 0, 0,       0,     /* 0 */
								 23,48, &patch,  0,     /* 1 patch */
								 21,53, &foots,  0,     /* 2 footswitch mode */
                         24,52, &keyalg, 0,     /* 3 keyboard algorithm */
                         22,59, 0,       1,     /* 4 detune */
                         22,69, 0,       0,     /* 5 output select */

                                              /* voice 'A' parameters: */

                         22,20, 0,       1,     /* 6 glide rate */
                         22, 8, &glide,  3,     /* 7 glide shape */
                          8, 8, &sweepm, 0,     /* 8 sweep mode */
                          9, 8, 0,       0,     /* 9 sweep rate */
                         11, 8, &ratmod, 0,     /* 10 sweep rate mod */
								 10, 8, &swave,  0,     /* 11 sweep wave shape */
								 12, 8, &samod,  0,     /* 12 sweep ampl mod */
                          2, 8, &touch,  0,     /* 13 env 1 ampl touch */
                          3, 8, 0,       0,     /* 14 env 1 attack */
								  3,11, &envmod, 0,     /* 15 env 1 attack mod */
								  4, 8, 0,       0,     /* 16 env 1 decay */
								  4,11, &envmod, 0,     /* 17 env 1 decay mod */
                          5, 8, 0,       0,     /* 18 env 1 release */
                          6,49, 0,       0,     /* 19 env 2 delay */
                          2,49, &touch,  0,     /* 20 env 2 ampl touch */
                          3,49, 0,       0,     /* 21 env 2 attack */
                          3,52, &envmod, 0,     /* 22 env 2 attack mod */
                          4,49, 0,       0,     /* 23 env 2 decay */
                          4,52, &envmod, 0,     /* 24 env 2 decay mod */
                          5,49, 0,       0,     /* 25 env 2 release */
                         15, 8, 0,       0,     /* 26 osc tune */
                         16, 8, &mod,    2,     /* 27 osc mod 1 select */
                         16,19, 1,       1,     /* 28 osc mod 1 depth */
								 17, 8, &mod,    2,     /* 29 osc mod 2 select */
								 17,19, 1,       1,     /* 30 osc mod 2 depth */
                         18, 8, &mod,    2,     /* 31 osc mod 3 select */
                         18,19, 1,       1,     /* 32 osc mod 3 depth */
								 19, 8, &wave,   0,     /* 33 osc wave shape */
								 20, 8, 0,       0,     /* 34 osc pulse width */
								 21, 8, &mod,    2,     /* 35 osc width mod select */
                         21,19, 1,       1,     /* 36 osc width mod depth */
                          9,49, &filter, 0,     /* 37 filter mode */
                         11,49, 0,       1,     /* 38 filter resonance */
                         10,49, 0,       0,     /* 39 filter tune */
                         12,49, &mod,    2,     /* 40 filter mod 1 select */
                         12,60, 1,       1,     /* 41 filter mod 1 depth */
                         13,49, &mod,    2,     /* 42 filter mod 2 select */
                         13,60, 1,       1,     /* 43 filter mod 2 depth */
                         14,49, &mod,    2,     /* 44 filter mod 3 select */
                         14,60, 1,       1,     /* 45 filter mod 3 depth */
                         17,49, &amp,    2,     /* 46 amplifier mod 1 select */
								 17,61, 0,       1,     /* 47 amplifier mod 1 depth */
								 18,49, &amp,    2,     /* 48 amplifier mod 2 select */
                         18,61, 0,       1,     /* 49 amplifier mod 2 depth */
                         19,49, &ampmod, 0,     /* 50 amplifier mod 3 select */
								  0, 0, 0,       0,     /* 51 link */
								  0, 0, 0,       0,     /* 52 edit */
								  0, 0, 0,       0,     /* 53 keyboard spilt */
                          0, 0, 0,       0,     /* 54 main transpose */
                          0, 0, 0,       0,     /* 55 link transpose */
                                
                                               /* voice 'B' parameters: */

                         22,36, 0,       1,     /* 56 glide rate */
                         22,24, &glide,  3,     /* 57 glide shape */
                          8,24, &sweepm, 0,     /* 58 sweep mode */
                          9,24, 0,       0,     /* 59 sweep rate */
                         11,24, &ratmod, 0,     /* 60 sweep rate mod */
                         10,24, &swave,  0,     /* 61 sweep wave shape */
								 12,24, &samod,  0,     /* 62 sweep ampl mod */
								  2,24, &touch,  0,     /* 63 env 1 ampl touch */
                          3,24, 0,       0,     /* 64 env 1 attack */
                          3,27, &envmod, 0,     /* 65 env 1 attack mod */
								  4,24, 0,       0,     /* 66 env 1 decay */
								  4,27, &envmod, 0,     /* 67 env 1 decay mod */
								  5,24, 0,       0,     /* 68 env 1 release */
                          6,65, 0,       0,     /* 69 env 2 delay */
                          2,65, &touch,  0,     /* 70 env 2 ampl touch */
                          3,65, 0,       0,     /* 71 env 2 attack */
                          3,68, &envmod, 0,     /* 72 env 2 attack mod */
                          4,65, 0,       0,     /* 73 env 2 decay */
                          4,68, &envmod, 0,     /* 74 env 2 decay mod */
                          5,65, 0,       0,     /* 75 env 2 release */
                         15,24, 0,       0,     /* 76 osc tune */
                         16,24, &mod,    2,     /* 77 osc mod 1 select */
                         16,35, 1,       1,     /* 78 osc mod 1 depth */
                         17,24, &mod,    2,     /* 79 osc mod 2 select */
								 17,35, 1,       1,     /* 80 osc mod 2 depth */
								 18,24, &mod,    2,     /* 81 osc mod 3 select */
                         18,35, 1,       1,     /* 82 osc mod 3 depth */
                         19,24, &wave,   0,     /* 83 osc wave shape */
								 20,24, 0,       0,     /* 84 osc pulse width */
								 21,24, &mod,    2,     /* 85 osc width mod select */
								 21,35, 1,       1,     /* 86 osc width mod depth */
                          9,65, &filter, 0,     /* 87 filter mode */
                         11,65, 0,       1,     /* 88 filter resonance */
                         10,65, 0,       0,     /* 89 filter tune */
                         12,65, &mod,    2,     /* 90 filter mod 1 select */
                         12,76, 1,       1,     /* 91 filter mod 1 depth */
                         13,65, &mod,    2,     /* 92 filter mod 2 select */
                         13,76, 1,       1,     /* 93 filter mod 2 depth */
                         14,65, &mod,    2,     /* 94 filter mod 3 select */
                         14,76, 1,       1,     /* 95 filter mod 3 depth */
                         17,65, &amp,    2,     /* 96 amplifier mod 1 select */
                         17,76, 0,       1,     /* 97 amplifier mod 1 depth */
								 18,65, &amp,    2,     /* 98 amplifier mod 2 select */
								 18,76, 0,       1,     /* 99 amplifier mod 2 depth */
                         19,65, &ampmod, 0};    /* 100 amplifier mod 3 select */


int dparam;                    /* display parameter number */

void dispprg( int init_params )            /* initiate display entire program */
{
       int changed_parms, parameter;
       static char last_inp_msg[60];
       extern char inp_msg[];

		 textcolor(LIGHTCYAN);
       changed_parms = 0;

       for ( dparam=0; dparam< 101; ++dparam )  {

           /* changed params only in patch mode
              always display what is changing 
              but don't display anything that did not change */

           if ( init_params == CHANGED_PARAMS ) { 
			      if ( parext( dparam, inp_msg) != parext( dparam, last_inp_msg )){
                   /* get the parameter and display */
                   parameter = parext( dparam, inp_msg );    
					    disppar( dparam, parameter, DISPLAY );
               }
           }

           /* do blanking as needed when displaying whole programs */
			  else { 

               parameter = parext( dparam, inp_msg );    /* get the parameter */ 

               /* when displaying whole programs, 
                  do not display unused modulation sources and depths */

               if ( par_crt_table[dparam].blank == 0 ||

                   (par_crt_table[dparam].blank == 1 && parameter != 0) ||

                   (par_crt_table[dparam].blank == 2 && 
                       parext( dparam+1, inp_msg ) != 0) ||

                   (par_crt_table[dparam].blank == 3 && 
                       parext( dparam-1, inp_msg ) != 0) ) {

							 disppar( dparam, parameter, DISPLAY );
							 ++changed_parms;
					}
               else {
							 disppar( dparam, parameter, BLANK );
							 ++changed_parms;
					}
           }
		 }

		 if (changed_parms > 2)  /* if program changes leave cursor on patch */
					disppar( 1, parext( 1, inp_msg ), DISPLAY );

                                /* update last message to test for changes */
       memcpy( last_inp_msg, inp_msg, 60);
}



int link_flag;



void disppar( int par, int val, int blanking )  /* display parameter 
                                                   given parameter number
												               and value */

{
       int i, parinx, length;
       char *ptr;

       if ( par == 51 )
               link_flag = val & 0xC0;

		 if ( ! par_crt_table[par].row )
               return;

       at( par_crt_table[par].row, par_crt_table[par].column - 1 );

		 putchar(' ');           /* leading blank */

       if ( (par==16 || par==66) && val==31 ) {     /* special exceptions */
					printf("99");                           /*env 1 decay*/
					return;
		 }
       if ( (par==18 || par==68) && val==31 ) {        /*env 1 release*/
               printf("threshold");
               return;
       }
       if ( (par==19 || par==69) && val==31 ) {        /*env 2 delay*/
               printf("sweep trigger");
               return;
		 }
       if ( (par==23 || par==73) && val==31 ) {        /*env 2 decay*/
               printf("99");
               return;
		 }
		 if ( (par==25 || par==75) && val==31 ) {        /*env 2 release*/
               printf("threshold");
               return;
		 }

		 switch ((int)par_crt_table[par].xlate



) {

               case 0:
                       if (blanking==DISPLAY)
                               printf( "%d", val );
                       else
                               printf( "  " );
							  break;

               case 1:
                       if (blanking==DISPLAY) {
                               if ( val<0 )
										         putchar( '-' );
							          else
                                       putchar( '+' );
                               printf( "%d", abs( val ) );
                       }
                       else
                               printf( "   " );
							  break;

					default:
                       ptr = par_crt_table[par].xlate;
                       parinx = 0;

                       while ( parinx < val ) {
                               if ( ! *ptr )
                                       ++ parinx ;
                               ++ ptr;
							  }
                       if (blanking==DISPLAY) 
                               cputs( ptr );
                       else {
                               length = strlen( ptr );
                               for ( i=0; i<length; ++i )  
                                       printf( " " );
                       }
                       break;

		 }

       putchar ( ' ' );

		 /* additional blank pads for to compliment special cases above */

       if ( par==18 || par==68 || par==25 || par== 75 )
               printf("       ");
       if ( par==19 || par==69 )
               printf("           ");


       at( par_crt_table[par].row, par_crt_table[par].column );

}




char m1[10] = { "S W E E P" };
char m2[5] = { "Mode" };
char m3[5] = { "Rate" };
char m4[6] = { "Shape" };
char m5[6] = { "Rate" };
char m6[6] = { "Ampl" };
char m7[20] = { "O S C I L L A T O R" };
char m8[5] = { "Tune" };
char m9[3] = { "1" };
char m10[3] = { "2" };
char m11[3] = { "3" };
char m12[6] = { "Shape" };
char m13[6] = { "Width" };
char m14[7] = { "Width" };
char m15[6] = { "PATCH" };
char m16[23] = { "E N V E L O P E  O N E" };
char m17[6] = { "Touch" };
char m18[5] = { "A A" };
char m19[5] = { "D D" };
char m20[2] = { "R" };
char m21[12] = { "F I L T E R" };
char m22[5] = { "Mode" };
char m23[4] = { "Res" };
char m24[6] = { "Glide" };
char m25[10] = { "ALGORITHM" };
char m26[7] = { "Output" };
char m27[23] = { "E N V E L O P E  T W O" };
char m28[6] = { "Delay" };
char m29[7] = { "Detune" };
char m30[18] = { "A M P L I F I E R" };
char m31[11] = { "Footswitch" };

char l1[41] = { "͹" };
char l2[42] = { "" };
char l3[2] = { "" };


char bd[4] = { '.' };                    /* begin dim */
char ed[4] = { '.' };                    /* end dim   */





struct {
       int row;
       int column;
       char *message;
} crtlabels[NUMLABELS] = {
                      7, 1,&m1,
                      8, 1,&m2,
                      9, 1,&m3,
							10, 1,&m4,
							11, 1,&m5,
                     12, 1,&m6,
                     14, 1,&m7,
							15, 1,&m8,
							16, 1,&m9,
							17, 1,&m10,
                     18, 1,&m11,
                     19, 1,&m12,
                     20, 1,&m13,
                     21, 1,&m14,
                     23,42,&m15,
                      1, 1,&m16,
                      2, 1,&m17,
							 3, 1,&m18,
                      4, 1,&m19,
                      5, 1,&m20,
                      8,42,&m21,
							 9,42,&m22,
							10,42,&m8,
                     11,42,&m23,
                     12,42,&m9,
							13,42,&m10,
							14,42,&m11,
							22, 1,&m24,
                     24,42,&m25,
                     22,62,&m26,
                      1,42,&m27,
                      2,42,&m17,
                      3,42,&m18,
                      4,42,&m19,
                      5,42,&m20,
							 6,42,&m28,
                     22,52,&m29,
                     16,42,&m30,
                     17,42,&m9,
							18,42,&m10,
							19,42,&m11,
                     21,42,&m31,
                      6, 1,&l1,
							13, 1,&l1,
							23, 1,&l1,
							 7,40,&l2,
                     15,40,&l2,
                     20,40,&l2,
                      1,40,&l3,
                      2,40,&l3,
                      3,40,&l3,
                      4,40,&l3,
                      5,40,&l3,
							 8,40,&l3,
                      9,40,&l3,
                     10,40,&l3,
                     11,40,&l3,
							12,40,&l3,
							14,40,&l3,
                     16,40,&l3,
                     17,40,&l3,
							18,40,&l3,
							19,40,&l3,
							21,40,&l3,
                     22,40,&l3,
                     24,40,&l3,  };




void dispmask()                /* display CHROMA program mask */

{
		 int i;
		
		 clear_crt();            /* clear crt */
		 textcolor(WHITE);
		 
		 for ( i=0; i<NUMLABELS; ++i ) {
					if (i==42)
						textcolor(CYAN);
					at( crtlabels[i].row, crtlabels[i].column );
					cputs( crtlabels[i].message );
		 }
		 at(24,1);                /* leave cursor on dialogue line */
		 
}



struct {
		 char offset;                     /* offset into program buffer     */
		 char shift;                      /* number of bits to shift right  */
		 char mask;                       /* AND data mask                  */
		 char biflag;                     /* bipolar flag  ( 1 = bipolar    */
													 /*                 0 = unsigned ) */

} pardef[102] = { 31,0,0xFF,0,         /*0*/
						 1,0,0x0F,0,         /*1*/
						 5,0,0x07,0,         /*2*/
						31,4,0x0F,0,         /*3*/
						 2,3,0x1F,0,         /*4*/
                   2,1,0x03,0,         /*5*/
                  28,3,0x1F,0,         /*6*/
                  14,6,0x01,0,         /*7*/
                   4,0,0x03,0,         /*8*/
                   4,2,0x3F,0,         /*9*/
                   3,4,0x0F,0,         /*10*/
                   6,4,0x0F,0,         /*11*/
                   6,0,0x0F,0,         /*12*/
                   9,0,0x07,0,         /*13*/
						 7,3,0x1F,0,         /*14*/
                   7,0,0x07,0,         /*15*/
                   8,3,0x1F,0,         /*16*/
                   8,0,0x07,0,         /*17*/
                   9,3,0x1F,0,         /*18*/
                  10,3,0x1F,0,         /*19*/
                  13,0,0x07,0,         /*20*/
                  11,3,0x1F,0,         /*21*/
						11,0,0x07,0,         /*22*/
                  12,3,0x1F,0,         /*23*/
                  12,0,0x07,0,         /*24*/
                  13,3,0x1F,0,         /*25*/
                  14,0,0x3F,0,         /*26*/
                  18,4,0x0F,0,         /*27*/
                  15,0,0x7F,1,         /*28*/
                  18,0,0x0F,0,         /*29*/
                  16,0,0x7F,1,         /*30*/
                  19,4,0x0F,0,         /*31*/
						17,0,0x7F,1,         /*32*/
                  20,0,0x03,0,         /*33*/
                  20,2,0x3F,0,         /*34*/
                  19,0,0x0F,0,         /*35*/
                  21,0,0x7F,1,         /*36*/
                  15,7,0x01,0,         /*37*/
                  10,0,0x07,0,         /*38*/
                  22,0,0x3F,0,         /*39*/
						26,4,0x0F,0,         /*40*/
                  23,0,0x7F,1,         /*41*/
                  26,0,0x0F,0,         /*42*/
                  24,0,0x7F,1,         /*43*/
                  27,4,0x0F,0,         /*44*/
                  25,0,0x7F,1,         /*45*/
                  27,2,0x03,0,         /*46*/
                   3,0,0x0F,0,         /*47*/
                  27,0,0x03,0,         /*48*/
                   5,4,0x0F,0,         /*49*/
						28,0,0x07,0,         /*50*/
                   0,0,0xFF,0,         /*51*/
                  30,0,0xFF,0,         /*52*/
                  32,0,0xFF,1,         /*53*/
                   1,6,0x03,0,         /*54*/
                   1,4,0x03,0,         /*55*/
                  58,3,0x1F,0,         /*56*/
                  44,6,0x01,0,         /*57*/
						34,0,0x03,0,         /*58*/
                  34,2,0x3F,0,         /*59*/
                  33,4,0x0F,0,         /*60*/
                  36,4,0x0F,0,         /*61*/
                  36,0,0x0F,0,         /*62*/
                  39,0,0x07,0,         /*63*/
                  37,3,0x1F,0,         /*64*/
                  37,0,0x07,0,         /*65*/
                  38,3,0x1F,0,         /*66*/
                  38,0,0x07,0,         /*67*/
						39,3,0x1F,0,         /*68*/
                  40,3,0x1F,0,         /*69*/
                  43,0,0x07,0,         /*70*/
                  41,3,0x1F,0,         /*71*/
                  41,0,0x07,0,         /*72*/
                  42,3,0x1F,0,         /*73*/
                  42,0,0x07,0,         /*74*/
                  43,3,0x1F,0,         /*75*/
						44,0,0x3F,0,         /*76*/
                  48,4,0x0F,0,         /*77*/
                  45,0,0x7F,1,         /*78*/
                  48,0,0x0F,0,         /*79*/
                  46,0,0x7F,1,         /*80*/
                  49,4,0x0F,0,         /*81*/
                  47,0,0x7F,1,         /*82*/
                  50,0,0x03,0,         /*83*/
                  50,2,0x3F,0,         /*84*/
                  49,0,0x0F,0,         /*85*/
						51,0,0x7F,1,         /*86*/
                  45,7,0x01,0,         /*87*/
                  40,0,0x05,0,         /*88*/
                  52,0,0x3F,0,         /*89*/
                  56,4,0x0F,0,         /*90*/
                  53,0,0x7F,1,         /*91*/
                  56,0,0x0F,0,         /*92*/
                  54,0,0x7F,1,         /*93*/
						57,4,0x0F,0,         /*94*/
                  55,0,0x7F,1,         /*95*/
                  57,2,0x03,0,         /*96*/
                  33,0,0x0F,0,         /*97*/
                  57,0,0x03,0,         /*98*/
                  35,4,0x0F,0,         /*99*/
                  58,0,0x07,0,        /*100*/
                  29,0,0xFF,0         /*101*/
};



parext(param, buffer)          /* extract parameter from display
                                  program message */

int param;
char buffer[];

{
       int ext;

       ext = buffer[ pardef[ param ].offset + 1 ]
               >> pardef[ param ].shift
               & pardef[ param ].mask;

       if ( pardef[ param ].biflag &&
            ext & ( (pardef[param].mask & 0xFF) >> 1  ^ pardef[param].mask))
               ext = ext | ~ pardef[ param ].mask;

       return (ext);
}



