/* cprog: CHROMA programmer main routine


>02/07/88       JLM: 1.0 original
>03/20/96       JLM: 1.1 Updated for BC4.5 & large model
>01/24/97       JLM: 1.2 Changed segment definitions in mpua.asm (ei DGROUP)
                         to upper case.  This matches the compiler and linker
                         defaults.  Otherwise mpua's data walks on the C data
                         segment - specifically into the patch descriptions.
>01/31/97       JLM: 1.3 Blank out noncontributing parameters

*/


#include <fcntl.h>
#include <stdio.h>
#include <sys\stat.h>
#include <conio.h>

/* CODES FOR MPU-401 COMMANDS */

#define CT_401_MODE_PLAY_STOP          0x05
#define CT_401_MODE_PLAY_START         0x0A
#define CT_401_MODE_REC_STOP           0x11
#define CT_401_MODE_OVERDUB_STOP       0x15
#define CT_401_MODE_RECORD_STANDBY     0x20
#define CT_401_MODE_REC_START          0x22
#define CT_401_MODE_OVERDUB_START      0x2A

#define CT_401_NO_ALL_NOTES_OFF        0x30
#define CT_401_NO_REAL_TIME            0x32
#define CT_401_THRU_DEAD               0x33
#define CT_401_COMMON_TO_HOST          0x38
#define CT_401_REAL_TIME_IN            0x39
#define CT_401_UART                    0x3F
#define CT_401_SET_REFA                0x40
#define CT_401_USE_INTERNAL_CLOCK      0x80
#define CT_401_USE_TAPE_CLOCK          0x81
#define CT_401_USE_MIDI_CLOCK          0x82
#define CT_401_METRONOME_ON_NORMAL     0x83
#define CT_401_METRONOME_OFF           0x84
#define CT_401_METRONOME_ON_DOUBLE     0x85
#define CT_401_BENDER_OFF              0x86
#define CT_401_BENDER_ON               0x87
#define CT_401_THRU_OFF                0x88
#define CT_401_THRU_ON                 0x89
#define CT_401_DATA_IN_STOP_OFF        0x8A
#define CT_401_DATA_IN_STOP_ON         0x8B
#define CT_401_INHIBIT_MEASURE_END     0x8C
#define CT_401_REQUEST_MEASURE_END     0x8D
#define CT_401_CONDUCTOR_OFF           0x8E
#define CT_401_CONDUCTOR_ON            0x8F
#define CT_401_RTM_AFFECT_OFF          0x90
#define CT_401_RTM_AFFECT_ON           0x91
#define CT_401_FSK_TO_INT              0x92
#define CT_401_FSK_TO_MIDI             0x93
#define CT_401_CLOCK_TO_HOST_OFF       0x94
#define CT_401_CLOCK_TO_HOST_ON        0x95
#define CT_401_SYS_EX_TO_HOST_OFF      0x96
#define CT_401_SYS_EX_TO_HOST_ON       0x97
#define CT_401_CREFA_OFF               0x98
#define CT_401_READ_RESET_RECORD_CTR   0xAB
#define CT_401_TEMPO_REQUEST           0xAF
#define CT_401_CLEAR_PLAY_COUNTERS     0xB8
#define CT_401_CLEAR_PLY               0xB9
#define CT_401_SET_CLOCK_RES           0xC2
#define CT_401_SEND_DATA               (0xD0 + 7)      /* send on track 7 */
#define CT_401_SEND_SYSTEM             0xDF
#define CT_401_SET_TEMPO               0xE0
#define CT_401_CLOCKS_PER_BEAT         0xE4
#define CT_401_BEATS_PER_MEASURE       0xE6
#define CT_401_CLOCK_TO_HOST_RATE      0xE7
#define CT_401_ACTIVE_TRACKS           0xEC
#define CT_401_ACCEPT_CHANNELS_LOW     0xEE
#define CT_401_ACCEPT_CHANNELS_HIGH    0xEF
#define CT_401_TIMING_OVERFLOW         0xF8
#define CT_401_NULL                    0xF8
#define CT_401_MEASURE_END             0xF9
#define CT_401_DATA_END                0xFC
#define CT_401_RESET                   0xFF

/* CODES FOR THE MESSAGES THAT ARE READ FROM THE 401 */

#define CF_401_REQUEST_PLAY_DATA       0xF0
#define CF_401_TIMING_OVERFLOW         0xF8
#define CF_401_DATA_END_MARK           0xFC
#define CF_401_ALL_END                 0xFC
#define CF_401_MEASURE_END             0xF9
#define CF_401_CONDUCT_REQ             0xF9
#define CF_401_CLOCK_TO_HOST           0xFD
#define CF_401_ACK                     0XFE
#define CF_401_SYSTEM_MSG              0xFF

#define ALL_PARAMS                     1
#define CHANGED_PARAMS                 0

#define TRUE                           1
#define FALSE                          0


/* function declarations */


void disp_file( char file[] );
void disp_dir( char file[] );
void disp_pnn( int number, char name[] );
void disp_mode( int program_flag );
void input_wait();
void clear25();
void disable_401();
void reset_int_vector();



struct {                               /* program names (global)*/
		 char name[25];
} progs[51];






main()
{

extern int input_flag,midi_program;
extern char inp_msg[];
static char req_midi[8] = {240, 8, 0, 75, 89, 0, 52, 247};
static char req_patch[8] = {240, 8, 0, 75, 89, 0, 0, 247};
static char trn_patch[128] = {240, 8, 0, 75, 89, 127, 0};
static char file[13] = { "DEFAULT.CHR" };
static char dotchr[5] = { ".CHR" };
FILE *channel;
char kin[80];
char *nameptr, *msgptr;
int handle;
int mask_flag,boot_flag, program_flag, patch_flag, midi_flag, key, i, j;
int local_midi_program, displayed_midi_program;
long int count;



printf("\nChroma    Programmer    PC\n");
printf(  "Version 1.3    31 Jan 1997\n\n");
printf("Press [Return] to continue.");

input_flag = 1;
input_flag = 0;

_fmode = O_BINARY;

program_flag = TRUE;
patch_flag = FALSE;
midi_flag = FALSE;
mask_flag = FALSE;
boot_flag = FALSE;
gets(kin);

	 /* get name of initial program file if any */
	 if ( (channel = fopen( "BOOT.CHR", "r" )) != 0 ) {
				nameptr = file;

				for ( i=0; i<14; i++, nameptr++ )
						  *nameptr = getc( channel );

				boot_flag = TRUE;
	 }

if (!initialize_shell())                       /* Set everything up */
	 {
	 printf("MIDI interface will not initialize.\n");
	 exit(0);
	 }

sh_send_command_401( CT_401_SYS_EX_TO_HOST_ON );

sh_send_command_401( CT_401_DATA_IN_STOP_ON );

/* sh_send_command_401( CT_401_THRU_ON ); */


input_flag = 0;

sh_write_req_401(CT_401_SEND_SYSTEM,           /* get midi parameters         */
	 &req_midi,                                 /* ( channel will be updated ) */
	 8 );

input_wait();

dispmask();                                    /* display parameter mask */
disp_file( file );                             /* and file name          */
disp_mode(program_flag);                       /* and mode               */

sh_write_req_401(CT_401_SEND_SYSTEM,           /* get patch 0 */
	 &req_patch,
	 8 );

input_wait();

dispprg(ALL_PARAMS);                            /* display program parameters */



while (TRUE) {                                                   /* main loop */


	 if (midi_flag) {
		  sh_write_req_401(CT_401_SEND_SYSTEM,
				&req_midi,
				8 );
		  input_wait();

																	/* display midi parameters */
	 }



	 if (patch_flag) {
		  sh_write_req_401(CT_401_SEND_SYSTEM,
				&req_patch,
				8 );
		  input_wait();

		  dispprg(CHANGED_PARAMS);                /* display program parameters */
	 }

		  local_midi_program = midi_program;
		  if (local_midi_program != displayed_midi_program) {
				sh_write_req_401(CT_401_SEND_SYSTEM,
					 &req_patch,
					 8 );
				input_wait();

				if (mask_flag) {
					 dispmask();                         /* display parameter mask */
					 disp_file( file );                  /* and file name          */
					 disp_mode(program_flag);            /* and mode               */

					 dispprg(ALL_PARAMS);            /* display program parameters */
					 mask_flag = FALSE;
				}
				else
					 dispprg(ALL_PARAMS);            /* display program parameters */

																	/* display name and number */
				disp_pnn (local_midi_program, progs[local_midi_program].name);
				displayed_midi_program = local_midi_program;
		  }




	 if ( kbhit() || boot_flag ) {              /* keyboard input */

		  if (boot_flag)
				key = 'L';                         /* force initial load */
		  else
				key = getch();


		  switch ( key ) {

				case 'Q':                          /* exit to DOS */
					 sh_send_command_401( CT_401_SYS_EX_TO_HOST_OFF );
					 sh_send_command_401( CT_401_DATA_IN_STOP_OFF );

					 disable_401();
					 reset_int_vector();
					 clear_crt();
					 exit(0);
					 break;


				case 'D':
					 if (mask_flag)  {
						 dispmask();                      /* display parameter mask */
						 disp_file( file );               /* and file name          */
						 disp_mode(program_flag);         /* and mode               */

						 sh_write_req_401(CT_401_SEND_SYSTEM,        /* get patch 0 */
												&req_patch,
												8 );

						 input_wait();

						 dispprg(ALL_PARAMS);         /* display program parameters */
						 mask_flag = FALSE;
					 }



					 else {
						  disp_dir( file );
						  mask_flag = TRUE;
					 }
					 break;


				case 'F':                          /* accept patch file name */
					 at( 25, 1 );
					 textcolor(WHITE);
					 printf( "Enter file name: " );
					 kin[0] = 9;
					 cgets( kin );
					 strupr( &kin[2] );
					 strcat( &kin[2], dotchr );
					 strcpy( file, &kin[2] );

					 disp_file( file );
					 clear25();
					 break;


				case 'P':                          /* input individual patch name */
					 at(25,1);
					 printf( "Enter patch name: " );
					 kin[0] = 24;
					 cgets( kin );
					 strcpy( progs[displayed_midi_program].name, &kin[2] );
					 disp_pnn( displayed_midi_program,
								  progs[displayed_midi_program].name );
					 clear25();
					 break;


			  case 'M':                           /* change mode */
					 patch_flag = !patch_flag;
					 program_flag = !program_flag;
					 disp_mode(program_flag);
					 break;

				case 'L':                          /* load file */
					 channel = fopen( file, "r" );
					 if ( !channel )
						  break;

					 at (25,1);
					 printf( "READING FILE %s", file );

					 nameptr = progs[0].name;       /* read names */
					 for ( i=0; i<51*25; i++, nameptr++ )
						  *nameptr =  getc (  channel );


						for ( i=1; i<51; i++ ) {     /* read each patch */
							 msgptr = &trn_patch[7];
							 trn_patch[6] = i;
							 trn_patch[125] = 247;

							 msgptr = &trn_patch[7];
							 for ( j=1; j<=59; j++, msgptr++ )  {
								  *msgptr = getc ( channel );
								  *(msgptr+1) = *msgptr;
																	  /* high nibble */
								  *msgptr = (*msgptr >> 4) & 0x0F;
								  msgptr++;
								  *msgptr = *msgptr & 0x0F;  /* low nibble */
							 }

							 sh_write_req_401(CT_401_SEND_SYSTEM,
								  &trn_patch,
								  126 );
						}


						fclose( channel );

					 /* save current file name for later automatic read at startup */
						handle = creat( "BOOT.CHR", S_IWRITE );
						channel = fdopen( handle, "w" );
						nameptr = file;
						for ( i=0; i<14; i++, nameptr++ )
							 putc( *nameptr, channel );
						fclose( channel );
						boot_flag = FALSE;

						clear25();
						break;

			  case 'S':                           /* save file */
					 remove("temp.chr");            /* original to temp.chr */
					 rename(file, "temp.chr");      /* in case trouble writing */	
					 handle = creat( file, S_IWRITE );
					 channel = fdopen( handle, "w" );
					 if ( !channel )
								break;

					 at (25,1);
					 printf( "WRITING FILE %s ", file );

					 nameptr = progs[0].name;       /* write patch names */
					 for ( i=0; i<51*25; i++, nameptr++ )
								putc( *nameptr, channel );

						for ( i=1; i<51; i++ ) {     /* request each patch */
							 req_patch[6] = i;
							 sh_write_req_401(CT_401_SEND_SYSTEM,
								  &req_patch,
								  8 );
							 input_wait();

							 msgptr = &inp_msg[1];    /* write each patch */
							 for ( j=1; j<=59; j++, msgptr++ )
								  putc( *msgptr, channel );
							 }

					 req_patch[6] = 0;
					 fclose( channel );
					 clear25();
					 break;
		  }


	 }
	 count = 0;
	 while (++count<5000);

}                                              /* end main loop */



}                                              /* end main() */




void disp_file( char file[] )

{
		  at( 24, 1 );  printf( "            " );
		  at( 24, 1 );  printf( "%s", file );
}




void disp_pnn( int number, char name[] )       /* display program number */
															  /*  and name              */

{

		  if (number!=0) {
				at (24,14);  printf ("                          ");
				at (24,14);  printf ("%d %s",number, name);
				at (24,14);
		  }
}



void disp_mode( int program_flag )

{
		  at( 21, 73 );
		  if (program_flag)
				printf("PROGRAM");
		  else
				printf("PATCH  ");
}



void clear25()

{

		 at( 25, 1);
		 printf( "                                                    " );
		 at( 25, 1);

}


void input_wait()

{
long int count;
extern int input_flag;

		 count = 0;

		 /* 500000 for 10MHz 286 */
		 while ( !input_flag && ++count<2000000);

		 if (!input_flag) {
			  textcolor(WHITE);
			  at (25,1);
			  printf ("CHROMA communications TIMEOUT");
		 }
		 else
			  input_flag = 0;
}


void disp_dir( char file[] )                       /* display patch directory */


{

int row, column, prog;

		 clear_crt();                                /* clear crt, home */
		 printf( "\n          Patch Directory for File %s \n\n\n", file );

		 for ( row=1; row<18; ++row ) {

					for ( column=0; column<3; ++column ) {

							  if (column == 1  || column == 2)
									printf(" ");

							  prog = column * 17 + row;
							  if ( prog > 50 ) break;
							  printf( "%22.22s %2d",
										 progs[prog].name,
										 prog );

					}
					printf( "\n" );
		 }
}
