;-----------------------------------------------------------------------------
; MPUA.ASM  -  LOW LEVEL INTERFACE ROUTINES FOR ROLAND MPU-401 MIDI INTERFACE |
;              INPUT DRIVER FOR CHROMA PROGRAMMER/LIBRARIAN MESSAGES          |
;                                                                             |
; Interfaces to routines contained in C routine MPUC.C                        |
; Borland C++  medium/large model                                             |               
;                                                                             |               
; 03/20/96     Updated to new compiler and from small model                   |               
; 01/24/97     Changed segment definitions (ie DGROUP) to upper case to match |               
;              linker and compilers default.  (mpua's data walked on C's data |               
;              segment)                                                               |               
;-----------------------------------------------------------------------------



MY_STACK_SIZE           equ     1120            ;make ISR stack this size
INT_401                 equ 2
CTRL_BRK_INT            equ 023h
BASE_ADDRESS_401        equ 330h
STATUS_PORT_401         equ BASE_ADDRESS_401 + 1
DATA_IN_PORT_401        equ BASE_ADDRESS_401 + 0
DATA_OUT_PORT_401       equ BASE_ADDRESS_401 + 0
COMMAND_PORT_401        equ BASE_ADDRESS_401 + 1
DATA_READY_401_MASK     equ 40h
DATA_AVAILABLE_401_MASK equ 80h

PC_INT_OFFSET           equ 8                   ;irq 0 is int 8 to 8086

INT_RESET_PORT          equ 20h                 ;port to send EOI command to
INT_RESET_CODE          equ 20h                 ;EOI command

PC_INTERRUPT_MASK_REGISTER equ 21h              ;port to mask interrupts


;********************
;**   macros       **
;********************

c_in    macro                   ;this macro sets up a stack frame for C
    push        bp              ;must save bp reg for 'c' progs
    mov bp, sp                  ;set up frame pointer
    sub sp,4                    ;allocate stack space for regs
    push di
    push si                     ;save register vars
    endm

c_out   macro                   ;This macro cleans up for return to 'C'
    pop si
    pop di
    mov sp, bp
    pop bp                      ;restore caller's frame pointer
    cld                         ;dirrection flag must be clear for MSC
    ret                         ;and return
    endm

push_all_regs   macro
    push        es
    push        ds
    push        dx
    push        cx
    push        bx
    push        si
    push        di
    push        ax
    push        bp
    endm

pop_all_regs    macro
    pop bp
    pop ax
    pop di
    pop si
    pop bx
    pop cx
    pop dx
    pop ds
    pop es
    endm

enable_401_mac macro                    ;this macro sets the int
                                        ;controller in the PC to enable
                                        ;the 401 interrupt
    in  al, PC_INTERRUPT_MASK_REGISTER  ;read in current int mask
    and al, 0ffh - (INT_401 shl 1)      ;clear the current bit
    out PC_INTERRUPT_MASK_REGISTER, al  ;and write it out
    endm

disable_401_mac        macro            ;disables irq2
    in  al, PC_INTERRUPT_MASK_REGISTER  ;read in current int mask
    or  al, INT_401 shl 1               ;set the bit for 401
    out PC_INTERRUPT_MASK_REGISTER, al  ;and write it out
    endm


read_int_status macro
    in  al, PC_INTERRUPT_MASK_REGISTER  ;read in current int mask
    endm

write_int_status macro
    out PC_INTERRUPT_MASK_REGISTER, al  ;and write it out
    endm

ack_int macro                           ;this macro acknowleges an
                                        ;interrupt to the int controller
                                        ;chip in the PC
    mov dx, INT_RESET_PORT
    mov al, INT_RESET_CODE
    out dx, al
    endm

dos_function_call macro number          ;Call Dos, function in AH
    mov ah, number
    int 21h
    endm

locate_int_vec macro                    ;Calls dos to read the old value of
                                        ;an int vector
    dos_function_call 35h
    endm

set_int_vec     macro                   ;Calls dos to set a new value
                                        ;for an int vector
    dos_function_call 25h
    endm

;is_data returns zero flag set if there is data from 401
;kills ax, dx

is_data macro
    mov dx, STATUS_PORT_401             ;DX = &status port
    in  al, dx                          ;read status port
    and al, DATA_AVAILABLE_401_MASK     ;is there data to read?
    endm

;**************************************************
;**  stack structures for routines               **
;**                                              **
;** used to access stack frame passed from 'C'   **
;**************************************************

stack           struc
bpstore         dw      ?
retn_addr       dw      ?               ;small model
more_retn_addr  dw      ?               ;medium model
arg1            dw      ?
arg2            dw      ?
arg3            dw      ?
arg4            dw      ?
arg5            dw      ?
arg6            dw      ?
stack           ends


;************************************************************
;**                                                        **
;** DATA GROUP DECLARATION                                 **
;**                                                        **
;************************************************************


DGROUP  group   _DATA
_DATA   segment word public 'DATA'
        assume  ds:DGROUP

        public _input_flag,_inp_msg
        public _midi_channel,_midi_program

;;; variables for input interrupt processing

inpind          dw ?                    ;input indicator
nibble_flag     db ?                    ;NZ = data transmitted nibblized
file            dw ?                    ;file number received
nibble_ct       dw ?                    ;nibble count (2=high, 1=low)
byte_ct         dw ?                    ;byte count (59 bytes)
patch_ct        dw ?                    ;patch count (50 patches)
_midi_channel   dw ?                    ;midi channel from midi param dump
_midi_program   dw ?                    ;midi program number
buffer_index    dw ?                    ;index into input buffer
_input_flag     dw ?                    ;input flag 
                db 'buffer'
_inp_msg        db 1000 dup(?)          ;input buffer

;                db 'debugx'
;debug_size      equ 15000
debug_index     dw ?
;debug           db debug_size dup(?)    ;save input history


; ---- The following variables are used to save all the old
;      interrupt vectors that we replace, so they can be
;      restored on program exit

int_24_seg_store        dw      ?
int_24_off_store        dw      ?
int_1b_seg_store        dw      ?
int_1b_off_store        dw      ?
int_401_seg             dw      ?       ;this is where the old interrupt vec...
int_401_offs            dw      ?       ;... is stored.  it is replaced by 401 int vector
int_break_seg           dw      ?
int_break_offs          dw      ?

; --- The following vars are used to manage a very small stack
;     that is used for interrupt enable/disable stacking

INT_STACK_SIZE          equ     8 * 1
int_stack               db      (INT_STACK_SIZE/8) dup ('intstack')
int_stack_pointer       dw      0

; --- The following vars are used to manage a stack that is used
;     by the interrupt service routine.  This is needed because
;     if DOS was interrupted by the 401, the stack will be in
;     a state where c routines can not be called.

mystack                 db      (MY_STACK_SIZE / 8) dup ('STACK   ')
                                        ;allocate the isr stack
stkend                  db      ?       ;coerce a label at the end of it
save_stack_seg          dw      ?       ;place to save old stack segment ...
save_stack_pointer      dw      ?       ;    ... and the old stack pointer

_DATA   ends


;************************************************************
;**                                                        **
;** CODE SEGMENT                                           **
;**                                                        **
;**                                                        **
;************************************************************

	 extrn _fatal_dos_handler:FAR   ;'C' function to do user
													 ; interface for DOS errors
_TEXT segment byte public 'CODE'
    assume cs:_TEXT

    public     _set_int_vector
    public     _reset_int_vector
    public     _disable_401
    public     _enable_401
    public     _sh_init_enable_401
    public     _write_data_401
    public     _read_401
    public     _unlock_inp_msg
    public     _input_401

;************************************************************
;**                                                        **
;** INTERRRUPT VECTOR SETUP MACROS                         **
;**                                                        **
;**                                                        **
;************************************************************

get_vec macro num, seg, off
        mov     al, num
        locate_int_vec          ;;read the current int number
        mov     seg, es
        mov     off, bx         ;;put the vector in storage
        endm

set_vec macro num, subr
        mov ax,_TEXT            ;;ax = seg offset
        mov     ds,ax
        mov     dx, offset subr
        mov     al, num
        set_int_vec             ;;put the address of isr in vector
        endm


;****************************************
;**       SET_INT_VECTOR               **
;****************************************

; Sets vector for 401 int to point to interrupt service routine
; Also sets vectors for bios ctrl break, bdos ctrl break, dos fatal error
; Ctrl-break must be intercepted, because returning to DOS with 401
; ints set up will cause devastation.
; Fatal error must be intercepted, otherwise dos errors will
; overwrite screen windows, which look terrible
        db '_set_int_vector'

_set_int_vector proc    far
        c_in
        push    es      ;save es

    ;next we will read and save all to old vector values,
    ;so we can restore them later
        get_vec  <INT_401 + PC_INT_OFFSET>,int_401_seg,int_401_offs
;       get_vec CTRL_BRK_INT, int_break_seg, int_break_offs, bx
        get_vec 01bh, int_1b_seg_store, int_1b_off_store
    ;bios ctrl break int handler
        get_vec 024h, int_24_seg_store, int_24_off_store
    ;dos fatal error

        pop     es              ;restore segment reg
        push    ds

    ;Set the vectors to their new values
        set_vec  <INT_401 + PC_INT_OFFSET>, isr
;       set_vec  CTRL_BRK_INT, dummy_handler
        set_vec  01bh,  dummy_handler
        set_vec  024h,  asm_fatal_dos_handler
        pop     ds
        c_out
_set_int_vector endp

;************************************************************
;** RESET_INT_VECTOR()                                     **
;** Sets int vectors to what they were before seq          **
;** was started                                            **
;**                                                        **
;************************************************************
        db '_reset_int_vector'

reset_vec macro num, seg, off
        mov     ds, seg
        mov     dx, off
        mov     al, num
        set_int_vec             ;;restore address of old isr, if any
        endm

_reset_int_vector proc far
        c_in
        push    ds

        reset_vec  <INT_401 + PC_INT_OFFSET>,int_401_seg,int_401_offs
;       reset_vec CTRL_BRK_INT, int_break_seg, int_break_offs, bx
        reset_vec 01bh, int_1b_seg_store, int_1b_off_store
        reset_vec 024h, int_24_seg_store, int_24_off_store

        pop     ds
        c_out
_reset_int_vector endp


;************************************************************
;**                                                        **
;** CTRL_BREAK_HANDLER                                     **
;** FATAL_DOS_HANDLER                                      **
;**                                                        **
;************************************************************

dummy_handler proc far
        iret
dummy_handler endp


asm_fatal_dos_handler proc far
        push_all_regs           ;save machine state
        push    di
        push    ax              ;push args for function
        mov     ax, DGROUP
        mov     ds, ax          ;address data segment
        mov     es, ax          ;and extra segmant
        call    _fatal_dos_handler ;handler(ax, di)
        pop     ax
        pop     ax
        pop_all_regs
        mov     al,0            ;tell DOS to ignore error
        iret
asm_fatal_dos_handler endp


;************************************************************
;** ISR                                                    **
;** This is the actual 401 int service routine             **
;** It sets up segments, etc..., then calls input routine  **
;************************************************************
		  db 'isr'

isr     proc    far

		  push_all_regs                           ;save machine state
		  mov     ax, DGROUP
		  mov     ds, ax                          ;address data segment
		  mov     es, ax                          ;and extra segmant

		  disable_401_mac
		  mov     save_stack_seg, ss              ;save the stack segment
		  mov     save_stack_pointer, sp          ;save the stack pointer
		  mov     bx, DGROUP
		  mov     ss, bx                          ;set up the new stack segment
		  mov     sp, (offset DGROUP:(stkend -2)) ;and new stack pointer

		do_another:
		  mov     dx, DATA_IN_PORT_401
		  in      al, dx                          ;read in the interrupt data
																; from 401
		  xor     ah, ah                          ;make integer
		  call    FAR PTR _input_401              ;and call the function

		common:
		  is_data                                 ;more data from 401?
		  jz      do_another                      ;go back if so

		both_out:
        mov     ax, save_stack_seg              ;restore the stack segment
        mov     ss, ax
        mov     sp, save_stack_pointer          ;and the stack pointer

        cli                                     ;turn off system ints
        enable_401_mac                          ;turn on 401
        ack_int                                 ;and acknowlege interrupt
        pop_all_regs                            ;restore machine state

      isr_out:
        iret                                    ;and leave

isr     endp


;------------------------------------------------------------------------------
; _input_401  -  process each input characters into messages
;                enter ax=input character
;                       buffer patch and midi parameter dumps and indicate
;                        to mpuc
;                extract program number from MIDI program change messages
;------------------------------------------------------------------------------

;;; input states:

is_complete     equ -1                  ;buffer locked, ignore input
is_header       equ 0                   ;expecting 401 system header
                                        ; or program change
is_sys_ex       equ 1                   ;expecting system exclusive command
is_fender_id    equ 2                   ;expecting fender id
is_zero         equ 3                   ;expecting zero
is_4b           equ 4                   ;expecting 4bh
is_conv_id      equ 5                   ;expecting chroma to midi converter id
is_dump         equ 6                   ;expecting patch dump
is_file         equ 7                   ;expecting file type
is_data_byte    equ 8                   ;expecting data byte
is_program_no   equ 9                   ;expecting MIDI program number


;;; input constants:

header          equ 00ffh               ;401 system header
sys_ex          equ 00f0h               ;system exclusive command
fender_id       equ 0008h               ;fender id
conv_id         equ 0059h               ;chroma to midi converter id
dump            equ 007fh               ;patch dump


        db '_input_401'

_input_401 proc far

        mov bx,inpind

;       mov si,debug_index              ; save characters and inpind for debug
;       cmp si,debug_size-10
;       ja debug_full
;       mov debug[si],al
;       inc debug_index
;       inc si
;       mov debug[si],bl
;       inc debug_index
;
;     debug_full:
        cmp bx,is_header
        jne ex_sys_ex

        no_running_status:
          cmp ax,header                 ;401 system header received?
          jne not_header

            inc bx
            jmp _input_401_x

      not_header:
        mov cx,_midi_channel            ;MIDI program change received?
        dec cx
        or cx,00C0h
        cmp ax,cx
        jne restart_inp1

            mov bx,is_program_no        ;yes, program number next
            jmp _input_401_x


      ex_sys_ex:
        cmp bx,is_sys_ex
        jne ex_fender_id

          cmp ax,sys_ex                 ;system exclusive command received?
          jne restart_inp1

            inc bx
            jmp _input_401_x


      ex_fender_id:
        cmp bx,is_fender_id
        jne ex_zero

          cmp ax,fender_id              ;fender id received?
          jne restart_inp1

            inc bx
            jmp _input_401_x


      ex_zero:
        cmp bx,is_zero
        jne ex_4b

          cmp al,0                      ;zero received?
          jne restart_inp1

            inc bx
            jmp _input_401_x


      ex_4b:
        cmp bx,is_4b
        jne ex_conv_id

          cmp al,4bh                    ;4b received?
          je inp_4b

        restart_inp1:
          jmp restart_inp               ;need long jump

          inp_4b:
            inc bx
            jmp _input_401_x


      ex_conv_id:
        cmp bx,is_conv_id
        jne ex_dump

          cmp ax,conv_id                ;converter id received?
          jne restart_inp1

            inc bx
            jmp _input_401_x


      ex_dump:
        cmp bx,is_dump
        jne ex_file

          cmp ax,dump                   ;patch dump received
          jne restart_inp1

            inc bx
            jmp _input_401_x


        ex_file:
          cmp bx,is_file
          jne ex_data

          mov file,ax                   ;store file number
          mov patch_ct,1                ;assume single patch
          cmp ax,51                     ;50 patches?
          jne single_patch

            mov patch_ct,50             ;yes, set count

        single_patch:
          mov nibble_flag,1             ;data sent nibblized (default-patches)
          mov byte_ct,59                ;59 bytes in each patch  ( default )
          cmp ax,52                     ;MIDI dump  ( indicated by program 52 )
          jne patch_dump

            mov nibble_flag,0           ;data sent as bytes (bit 7=0)
            mov byte_ct,19              ;19 bytes in MIDI dump

        patch_dump:
          mov nibble_ct,2               ;start with high nibble
          mov buffer_index,1            ;start at front of buffer
          inc bx
          jmp _input_401_x


      ex_data:
        cmp bx,is_data_byte
        jne ex_program_no

        mov si,buffer_index
        cmp nibble_flag,0               ;data is nibbles?
        jnz data_is_nibbles

          mov _inp_msg[si],al           ;store the data byte
          cmp si,3                      ;at midi channel byte?
            jne next_byte

          mov cx,ax                     ;yes, store
          inc cx
          mov _midi_channel,cx          ;(sent relative to 0)
          jmp short next_byte

        data_is_nibbles:                ;assemble nibbles
          mov cx,nibble_ct
          cmp cx,2                      ;high nibble?
          jne low_nibble

            shl ax,1                    ;yes, shift left
            shl ax,1
            shl ax,1
            shl ax,1
            mov _inp_msg[si],al         ;store high nibble
            jmp short next_nibble

        low_nibble:
          or _inp_msg[si],al            ;and in low nibble

        next_nibble:
          dec cx
          mov nibble_ct,cx
          jnz _input_401_x

          mov nibble_ct,2               ;this byte complete

        next_byte:
          inc buffer_index              ;next position in buffer
          dec byte_ct                   ;more bytes?
          jnz _input_401_x

          mov byte_ct,59                ;reset byte count
          dec patch_ct                  ;more patches?
          jnz _input_401_x

;          push file                     ;pass file number
;          call _deal_with_input_401     ;process one message
;          pop ax

           mov _input_flag,1

          jmp short restart_inp         ;do not lock input buffer
                                        ;( assume application will not
                                        ;  request anything else that
                                        ;  uses the buffer until it is
                                        ;  done with this message. )


      ex_program_no:
        cmp bx,is_program_no
        jne ex_complete

          cmp al,0
          jns got_program
          jmp no_running_status          ;hi order bit indicates

          got_program:
            mov _midi_program,ax        ;input state remains
            jmp short _input_401_x      ; ( allow for running status )


      ex_complete:
        cmp bx,is_complete
        je _input_401_x                 ;input complete, buffer locked
                                        ;if not is_complete, bad input status

      restart_inp:
        mov bx,is_header                ;restart input waiting for header


      _input_401_x:
        mov inpind,bx                   ;save input indicator
        ret

_input_401 endp


;------------------------------------------------------------------------------
; _unlock_inp_msg  -  unlock input message buffer to allow more input
;------------------------------------------------------------------------------

_unlock_inp_msg proc far

        c_in
        mov inpind,is_header            ;set input state to waiting for header
        c_out

_unlock_inp_msg endp



;************************************************************
;** DISABLE_401()                                          **
;** Called to disable 401 interrupts.  Stacks old int      **
;**  status before disabling, so that nesting is           **
;**  preserved.                                            **
;************************************************************
        db '_disable_401'

_disable_401 proc far
        cli
        read_int_status                 ; get int enble byte in al

        mov     dl, al                  ;move it to bl
        and     dl, INT_401 shl 1       ;extract the bit for our ints
        mov     bx, int_stack_pointer   ;get current stack pointer
        mov     int_stack[bx], dl       ;store out int status there
        inc     bx
        mov     int_stack_pointer, bx   ;write back new stack pointer value
        or      al, INT_401 shl 1       ;mask off int

        write_int_status                ;and write it back
        sti

      disable_out:
        ret

_disable_401 endp


;************************************************************
;** Enable_401()                                           **
;** Restores whatever the int status was before            **
;**  disable_401() was called.                             **
;** Note: Calls to enable and disable must be matched      **
;************************************************************
        db '_enable_401'

_enable_401 proc far

        cli
        read_int_status                 ; get int enbale byte in al
        and     al, 0ffh - (INT_401 shl 1)
                                        ; get all the bits except the one
                                        ; for out int
        mov     bx, int_stack_pointer   ;get int stack pointer
        dec     bx                      ;move it down
        mov     int_stack_pointer, bx   ;store it
        or      al, int_stack[bx]       ;or in int status with old one
        write_int_status                ;write back to int controller
        sti                             ;turn ints on now
        ret

_enable_401 endp



;************************************************************
;** SH_INIT_ENABLE_401()                                   **
;** Starts up the 401 interrupts                           **
;**                                                        **
;************************************************************
        db '_sh_init_enable'

_sh_init_enable_401 proc far
        c_in
        cli
        mov inpind,0                    ;clear input state
        mov debug_index,0
        enable_401_mac
        sti
        c_out
_sh_init_enable_401 endp



;************************************************************
;** WRITE_DATA_401(DATA)                                   **
;** sends a byte to the 401 data port,                     **
;**  waits for handshake                                   **
;**                                                        **
;************************************************************
        db '_write_data_401'


_write_data_401 proc far

        c_in
        mov bx, [bp].arg1               ;get data byte in bl reg
        call write_sub                  ;and call fast sub
        c_out                           ;leave

_write_data_401 endp



;************************************************************
;**                                                        **
;**   write_sub                                            **
;**   Writes data to 401.                                  **
;**   enter with data byte in BL
;**   ax, cx, cx destroyed                                 **
;************************************************************

write_sub proc 
        mov dx, STATUS_PORT_401         ;set up pointer to 401 port

      write_clear_loop:
        in      al, dx
        and al, DATA_READY_401_MASK
        jnz write_clear_loop

        dec     dx                      ;point to data reg
        mov     al, bl
        out     dx, al                  ;send out the data

      write_out:
        ret
write_sub endp



;************************************************************
;** read_401()                                             **
;** reads a byte from the 401 data port                    **
;** waits for handshake                                    **
;**                                                        **
;************************************************************
        db '_read_401'

_read_401 proc far
        c_in

      read_clear_loop:
        mov dx, STATUS_PORT_401         ;DX = &status port
        in      al, dx
        and al, DATA_AVAILABLE_401_MASK
        jnz read_clear_loop

        dec     dx                      ;point to data reg
        in      al, dx
        xor     ah, ah

      read_out:
        c_out

_read_401 endp



_TEXT   ends
        end
